\name{simReads}
\alias{simReads}
\alias{CNVariant}
\alias{cnvLikelihood}
\alias{cnSet}
\alias{vtSet}
\title{Simulating Read-Counts and Working With Priors for DeepCNV}
\description{
  The \code{DeepCNV} class is used to fit a Bayesian model to targeted
  sequencing data from one or a few genes in order to draw inferences
  about possible copy number changes.  It includes routines to simulate
  read-counts with known copy number state and known fraction of normal
  'contaminating' cells.
}
\usage{
simReads(nMut, nVar, nu, cnstate=cnSet, depth=100, sdepth=25, \dots)
CNVariant(nu, S = cnSet, V = vtSet, M = 2)
cnvLikelihood(nu, K, N, V, S)
}
\arguments{
  \item{nMut}{integer; the number of somatic mutations in a gene}
  \item{nVar}{integer; the number of variant SNPs in a gene}
  \item{nu}{numeric between 0 and 1; the fr4action of normal cells in
    the sample}
  \item{cnstate}{the copy number state of the gene; must be one of
    "Deleted", "Normal", or "Gained", as enumerated in \code{cnSetg}}
  \item{depth}{integer; the average read depth at the gene}
  \item{sdepth}{integer; the stnadard deviation of the read depth across
  varaints within a gene}
  \item{\dots}{extra parameters to pass from \code{simReads} to \code{CNVariant}}
  \item{S}{The copy number state, as enumerated in \code{cnSet}}
  \item{V}{The variant type.  Must be one of "Mutation" or "SNP" as
    enumerated in \code{vtSet}}
  \item{M}{the total number of replicate copies of a (allelic) gene. The
    default value of 2 corresponds to a gain of one copy}
  \item{K}{Number of variant reads}
  \item{N}{Number of total reads (both variant and refernce)}
}
\details{
  The \code{DeepCNV} class is used to fit a Bayesian model to targeted
  sequencing data from one or a few genes in order to draw inferences
  about possible copy number changes. Basically, we assume that the
  observed data consists of a list  of triples (K, N, V), one for each
  variant in a gene.  Here K is the number of variant reads, N is the
  total number of reads, and V is the type of each variant (either a
  known SNP or a somatic mutation).  We model (K, N) using a binomial
  distribution, where the 'success' parameter \eqn{\phi} depends (in a
  deterministic way) on the unknown parameters of interest: the fraction
  \eqn{\nu} of normal cells in the sample and the copy number state (Normal,
  Deleted, or Gained).

  The functions \code{cnvLikelihood} and \code{CNVariant} are used to
  compute the log-likelihood of the unknown parameters given the
  observed data.  \code{CNVariant} computes the success parameter
  \eqn{\phi} as a function of the observed data (K, N, V), and this
  parameter is then used to compute the binomial log-likelihood.

  The \code{simReads} function generates simulated read-count data based
  on the underlying theoretical binomial model.  More details can be
  found in the vignettes \code{d01-cnvTheory} and
  \code{d02-oneGeneSims}.
}
\value{
  The \code{simReads} function returns a data frame suitable for use by
  the function \code{makeCNVPosterior}.
  
  The \code{CNVariant} function returns a real number between zero and
  one, corresponding to the fraction of reads that are expected to be
  variants given the variant type (\code{V}), the copy number state
  (\code{S}), and the fraction of normal cells (\code{nu}).

  The \code{cnvLikelihood} function returns a real number representing
  the log-likelihood (yes, I know; it probably should be renamed) of the
  parameters (S, \eqn{\nu}) given the observed data (K, N).
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com}
}
\seealso{
  \code{\link{CNVPrior}}, \code{\link{CNVPosterior}}.
}
\examples{
simReads(nMut=2, nVar=7, nu=0.17, "Norm", depth=130 )
# check log-likelihhoods of different copy number states
# for the same observed data
obs <- data.frame(K=c(69, 48), N=c(153, 167))
cnvLikelihood(0.22, obs)
cnvLikelihood(0.22, obs)
cnvLikelihood(0.22, obs)
}
\keyword{models}
\keyword{datagen}

